package net.lenkaspace.robotForaging.model;

import net.lenkaspace.creeper.model.CRBaseSituatedModel;
import net.lenkaspace.creeper.view.CRRenderer;
import net.lenkaspace.creeper.vo.CRVector3d;

public class Deposit extends CRBaseSituatedModel {

	public class Packet {
		public double grossAmount = 0; //resource + waste
		public double netAmount = 0; //pure resource
	}
	
	private double resourceConcentration; // how good this target is - range [0,1]
	private double resourceAmount; // how much resource there is. A robot extract 1 unit per trip
	private World world;
	
	public Deposit(double resourceAmount_, double resourceConcentration_, int id_, CRVector3d position_, CRVector3d size_, World world_) {
		super(id_, position_, size_, 0, CRBaseSituatedModel.SHAPE.CIRCLE, CRRenderer.CR_BLUE_CIRCLE);
		resourceAmount = resourceAmount_;
		resourceConcentration = resourceConcentration_;
		world = world_;
	}
	
	public Packet extractResource(double extractionRate) {
		Packet packet = new Packet();
		double toRet = 0;
		if (resourceAmount > 0) {
			resourceAmount -= extractionRate;
			toRet = extractionRate;
		}
		if (resourceAmount <= 0) {
			//-- remove self if empty
			world.removeSituatedModel(this);
		}
		packet.grossAmount = toRet;
		packet.netAmount = toRet * resourceConcentration;
		return packet;
	}
	
	public double getQuantity() {
		return resourceAmount;
	}
	
	public void setResourceConcentration(double c_) { 
		resourceConcentration = c_;
		if (resourceConcentration > 1) { resourceConcentration = 1; }
		else if (resourceConcentration < 0) { resourceConcentration = 0; }
		toolTip = String.valueOf(resourceConcentration);
		alpha = resourceConcentration;
	}
	public double getResourceConcentration() { return resourceConcentration; }

}
