package net.lenkaspace.robotForaging.model;

import net.lenkaspace.creeper.model.CRParameters;

public class Parameters extends CRParameters {
	
	//==================================== SINGLETON ====================================
    private static Parameters singletonReference;
	
	public static Parameters getSingleton()	  {
	  if (singletonReference == null) {
		  singletonReference = new Parameters();
		  //-- set all variables from the CRParameters
		  CRParameters origParams = CRParameters.getSingleton();
		  singletonReference.scenario = origParams.scenario;
		  singletonReference.reportFolderName = origParams.reportFolderName;
		  singletonReference.numOfTrials = origParams.numOfTrials;
		  singletonReference.numOfRuns = origParams.numOfRuns;
		  singletonReference.trialDuration = origParams.trialDuration;
		  singletonReference.setController(origParams.getController());
		  //-- rewrite CRParameters reference
		  CRParameters.setSingletonReference(singletonReference);
	  }
	  return singletonReference;
	}
	
	//================
	
	public int neighbourhoodSearchRadius = -1; //radius of neighborhood search
	public int neighbourhoodSearchTime = -1; //time [in seconds] to search in neighbourhood
	public int numOfRobots = -1;
	public int numOfDeposits = -1;
	public double totalResourceAvailable = -1;
	public int numOfDepositQualityGroups = -1;  // how many centres of high concentration there are
	public double depositQualityVariance = -1;  // variance of quality gaussian with middle in a group centre, value between [0,1], in percentage of half world width
	
	public double odomSegmentLength = -1;
	public double odomVarianceCoeff = -1; //values between [0,1], for error in location, Sigma^2 = odomVarianceCoeff*|odomSegmentLength|
	
	public int communicationRadius = -1; //how far a robot can broadcast 
	public int timeToRandWalk = -1;		//how long a B-Swarm robot performs random walk before it returns to base to pick up communications
	public int nearDistanceToBase = -1; //when robots come back to base, how close do they remain to it to pick up communications
	public int timeToWalkNearBase = -1; //how long a B-Swarm robot performs walk near base before it resumes random walk 
	
	public boolean useBSwarm = true;
	public boolean useNeighborhoodSearch = true;
	
	public Parameters() {
		super();	
	}
	
	/**
	 * Generate a string that lists all parameters. Subclasses should override this
	 * to add their own parameters.
	 */
	public String toString() {
		String content = super.toString();
		content += "\nNumber of robots: " + numOfRobots + "\n";
		content += "Use B-Swarm: " + String.valueOf(useBSwarm) + "\n";
		content += "Use neighborhood search: " + String.valueOf(useNeighborhoodSearch) + "\n";
		
		content += "\nNumber of deposits: " + numOfDeposits + "\n";
		content += "Deposit quality groups: " + numOfDepositQualityGroups + "\n";
		content += "Deposit quality variance: " + depositQualityVariance + "\n";
		
		content += "\nOdometry variance coefficient: " + odomVarianceCoeff + "\n";
		content += "Communication radius: " + communicationRadius + "\n";
		content += "Time to random walk: " + timeToRandWalk + "\n";
		content += "Time to walk near base:" + timeToWalkNearBase + "\n";
		
		content += "\nNeighborhood search radius: " + neighbourhoodSearchRadius + "\n";
		content += "Neighborhood search time: " + neighbourhoodSearchTime + "\n";
		content += "Near distance to base: " + nearDistanceToBase + "\n";
		
		//content += "Total resource available: " + totalResourceAvailable + "\n";
		content += "Odometry segment length: " + odomSegmentLength + "\n";
			
		return content;
		
	}
	
	
	/**
	 * Check that all parameters have been set. Sublcasses should override this
	 * to add their own checks.
	 * @return
	 */
	public boolean checkSelf() {
		boolean ok = super.checkSelf();
		if (neighbourhoodSearchRadius < 0) {
			System.err.println("Parameters :: neighborhood search radius not set");
			ok = false;
		}
		if (neighbourhoodSearchTime < 0) {
			System.err.println("Parameters :: neighborhood search time not set");
			ok = false;
		}
		if (numOfRobots < 0) {
			System.err.println("Parameters :: number of robots not set ");
			ok = false;
		}
		if (numOfDeposits <= 0) {
			System.err.println("Parameters :: number of targets not set or 0");
			ok = false;
		}
		if (totalResourceAvailable < 0) {
			System.err.println("Parameters :: total resource available no set");
			ok = false;
		}
		if (depositQualityVariance < 0) {
			System.err.println("Parameters :: deposit quality variance not set");
			ok = false;
		}
		if (numOfDepositQualityGroups < 0) {
			System.err.println("Parameters :: deposit quality groups not set");
			ok = false;
		}
		if (odomSegmentLength <= 0) {
			System.err.println("Parameters :: odometry segment length not set or 0");
			ok = false;
		}
		if (odomVarianceCoeff < 0) {
			System.err.println("Parameters :: odomerty variance coefficient not set");
			ok = false;
		}
		if (communicationRadius < 0) {
			System.err.println("Parameters :: communication radius not set");
			ok = false;
		}
		if (timeToRandWalk < 0) {
			System.err.println("Parameters :: time to random walk not set");
			ok = false;
		}
		if (nearDistanceToBase <= 0) {
			System.err.println("Parameters :: near distance to base not set or 0");
			ok = false;
		}
		if (timeToWalkNearBase < 0) {
			System.err.println("Parameters :: time to walk near base not set");
			ok = false;
		}
		
		return ok;
	}

}
