package net.lenkaspace.robotForaging.view;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.lenkaspace.creeper.CRController;
import net.lenkaspace.creeper.view.CRComponentFactory;
import net.lenkaspace.creeper.view.CRControlPanel;
import net.lenkaspace.robotForaging.model.Parameters;
import net.lenkaspace.robotForaging.model.Settings;
import net.lenkaspace.robotForaging.model.World;

@SuppressWarnings("serial")
public class ControlPanel extends CRControlPanel {
	
	protected JCheckBox bSwarmCheckBox;
	protected JTextField numOfDeposits;
	protected JTextField numOfQualityGroups;
	protected JTextField qualityGroupVariance;
	protected JTextField numOfRobots;
	
	public ControlPanel(Dimension size_, CRController controller_) {
		super(size_, controller_);
		
		this.setShowViewMovementControls(true);
		
		Settings settings = Settings.getSingleton();
		Parameters parameters = Parameters.getSingleton();
		
		//--- pre start simulation panel
		preStartSimulationPanel.setPreferredSize(new Dimension(preStartSimulationPanel.getWidth(), 70));
		
		CRComponentFactory.createJLabel("Deposits: ", preStartSimulationPanel);
		numOfDeposits = CRComponentFactory.createJTextField("100", 45, preStartSimulationPanel);
		
		CRComponentFactory.createJLabel("  Quality groups: ", preStartSimulationPanel);
		numOfQualityGroups = CRComponentFactory.createJTextField("0", 45, preStartSimulationPanel);
		
		CRComponentFactory.createJLabel("  Quality group variance: ", preStartSimulationPanel);
		qualityGroupVariance = CRComponentFactory.createJTextField("0.01", 45, preStartSimulationPanel);
		
		CRComponentFactory.createJLabel("Robots: ", preStartSimulationPanel);
		numOfRobots = CRComponentFactory.createJTextField("50", 45, preStartSimulationPanel);
		
		bSwarmCheckBox = CRComponentFactory.createJCheckBox("Use B-Swarm",parameters.useBSwarm, new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				Parameters.getSingleton().useBSwarm = ((JCheckBox)e.getSource()).isSelected();
				
			}
		}, preStartSimulationPanel);
		
		CRComponentFactory.createDividerJPanel(-1, 5, this);
		
		//--- movement panel
		
		CRComponentFactory.createJButton("Center at base", new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				controller.getRenderer().centerAtViewPosition(((World)controller.getWorld()).getBase().getPosition());
				
			}
		}, this);
		
		JPanel movementPanel = CRComponentFactory.createFlowLayoutJPanel(-1, 60, this);
		CRComponentFactory.createJCheckBox("Show avoid radius", settings.showAvoidRadius, new ActionListener() {	
			@Override
			public void actionPerformed(ActionEvent e) {
				Settings.getSingleton().showAvoidRadius = ((JCheckBox)e.getSource()).isSelected();
				
			}
		}, movementPanel);
		
		CRComponentFactory.createJCheckBox("Show odometry position", settings.showOdometryPosition, new ActionListener() {	
			@Override
			public void actionPerformed(ActionEvent e) {
				Settings.getSingleton().showOdometryPosition = ((JCheckBox)e.getSource()).isSelected();
				
			}
		}, movementPanel);
		CRComponentFactory.createJCheckBox("Show estimated deposit position", settings.showEstimDepoPosition, new ActionListener() {	
			@Override
			public void actionPerformed(ActionEvent e) {
				Settings.getSingleton().showEstimDepoPosition = ((JCheckBox)e.getSource()).isSelected();
				
			}
		}, movementPanel);
		redisplayParameters();
	}
	
	
	/**
	 * Set all input fields according to parameters coded manually. Called when a simulation
	 * is started by code, not by the control panel. Subclasses should override this
	 * to update display of their own parameter values.
	 */
	public void redisplayParameters() {
		super.redisplayParameters();
		if (bSwarmCheckBox != null) {
			bSwarmCheckBox.setSelected(Parameters.getSingleton().useBSwarm);
		}
	}
	
	/**
	 * Called when Start Simulation button is pressed
	 * @param e ActionEvent action event
	 */
	protected void onStartSimulationClicked(ActionEvent e) {
		//-- set parameters
        Parameters parameters = Parameters.getSingleton();
        parameters.numOfRobots = Integer.parseInt(numOfRobots.getText());
        parameters.numOfDeposits = Integer.parseInt(numOfDeposits.getText());
        parameters.numOfDepositQualityGroups = Integer.parseInt(numOfQualityGroups.getText());
        parameters.depositQualityVariance = Double.parseDouble(qualityGroupVariance.getText());
      
		super.onStartSimulationClicked(e);
	}
}
